@@ -5,7 +5,6 @@ import json
 import errno
 import random
 
-from concurrent import futures
 import spur
 import spur.ssh
 import starboard
@@ -17,6 +16,7 @@ from .. import processes
 from .. import dictobj
 from ..request import request_machine, MachineRequest
 from . import networkconfig
+from .. import futures
 
 local_shell = spur.LocalShell()
 
@@ -57,7 +57,8 @@ class Provider(object):
             request = args[0]
         else:
             request = request_machine(*(["peachtree"] + list(args)), **kwargs)
-        network = self._networking.settings_for(request)
+        image = self._images.image(request.image_name)
+        network = self._networking.settings_for(image, request)
         machine = self._start_with_network_settings(request, network)
         
         config = self._guest_network_config_for(machine)
@@ -69,37 +70,37 @@ class Provider(object):
             
     def start_many(self, requests):
         # TODO: assert name of each request is unique
-        with futures.ThreadPoolExecutor(max_workers=4) as executor:
-            network = self._networking.start_network()
+        network = self._networking.start_network()
+        
+        def start(request):
+            image = self._images.image(request.image_name)
+            network_settings = network.settings_for(image, request)
+            return self._start_with_network_settings(request, network_settings)
+        
+        machines = list(futures.thread_map(start, requests))
+        
+        addresses = []
+        for index, machine in enumerate(machines):
+            request = requests[index]
             
-            def start(request):
-                network_settings = network.settings_for(request)
-                return self._start_with_network_settings(request, network_settings)
-            
-            machines = list(executor.map(start, requests))
-            
-            addresses = []
-            for index, machine in enumerate(machines):
-                request = requests[index]
-                
-                eth1_address = "192.168.0.{0}".format(1 + index)
-                netmask = "255.255.255.0"
-                with machine.root_shell() as root_shell:
-                    config = self._guest_network_config_for(machine)
-                    config.configure_internal_interface(
-                        root_shell, eth1_address, netmask
-                    )
-                    addresses.append((request.name, eth1_address))
-            
-            for machine in machines:
+            eth1_address = "192.168.0.{0}".format(1 + index)
+            netmask = "255.255.255.0"
+            with machine.root_shell() as root_shell:
                 config = self._guest_network_config_for(machine)
-                with machine.root_shell() as root_shell:
-                    for hostname, address in addresses:
-                        root_shell.run(self._append_host_command(
-                            address, hostname, config
-                        ))
-            
-            return MachineSet(machines)
+                config.configure_internal_interface(
+                    root_shell, eth1_address, netmask
+                )
+                addresses.append((request.name, eth1_address))
+        
+        for machine in machines:
+            config = self._guest_network_config_for(machine)
+            with machine.root_shell() as root_shell:
+                for hostname, address in addresses:
+                    root_shell.run(self._append_host_command(
+                        address, hostname, config
+                    ))
+        
+        return MachineSet(machines)
     
     def _guest_network_config_for(self, machine):
         image = self._images.image(machine.image_name)
@@ -124,6 +125,7 @@ class Provider(object):
             name=request.name,
             identifier=identifier,
             image_name=request.image_name,
+            ssh_internal_port=image.ssh_internal_port,
             # TODO: either re-couple network, or find a better way of
             # storing network details
             forwarded_ports=network.forwarded_ports,
@@ -254,17 +256,25 @@ class Images(object):
             ]
             
         operating_system_family = description.get("operatingSystemFamily", "linux")
+        ssh_internal_port = description.get("sshPort", 22)
             
-        return Image(disks, memory_size, users, operating_system_family)
+        return Image(
+            disks=disks,
+            memory_size=memory_size,
+            users=users,
+            operating_system_family=operating_system_family,
+            ssh_internal_port=ssh_internal_port,
+        )
 
 
-class Image(object):
-    def __init__(self, disks, memory_size, users, operating_system_family):
-        self.disks = disks
-        self.memory_size = memory_size
-        self.users = users
-        self.operating_system_family = operating_system_family
-    
+Image = dictobj.data_class("Image", [
+    "disks",
+    "memory_size",
+    "users",
+    "operating_system_family",
+    "ssh_internal_port",
+])
+
     
 def _default_data_dir():
     xdg_data_home = os.environ.get("XDG_DATA_HOME", os.path.expanduser("~/.local/share"))
@@ -276,6 +286,7 @@ MachineStatus = dictobj.data_class("MachineStatus",
         "identifier",
         "name",
         "image_name",
+        "ssh_internal_port",
         "forwarded_ports",
         "start_time",
         "timeout",
@@ -377,6 +388,7 @@ class QemuMachine(object):
         self._users = users
         self.name = status.name
         self.image_name = status.image_name
+        self.ssh_internal_port = status.ssh_internal_port
         self.identifier = status.identifier
         self._process_set = processes.from_dir(status.process_set_run_dir)
         self._forwarded_ports = status.forwarded_ports
@@ -398,19 +410,16 @@ class QemuMachine(object):
     def public_port(self, guest_port):
         return self._forwarded_ports.get(guest_port, None)
         
-    def hostname(self):
+    def external_hostname(self):
         return starboard.find_local_hostname()
         
     def users(self):
         return self._users
 
 
-_GUEST_SSH_PORT = 22
-
-
 class UserNetworking(object):
-    def settings_for(self, request):
-        forwarded_ports = _generate_forwarded_ports(request.public_ports)
+    def settings_for(self, image, request):
+        forwarded_ports = _generate_forwarded_ports(image, request.public_ports)
         return UserNetworkSettings(forwarded_ports, [])
         
     def start_network(self):
@@ -422,8 +431,8 @@ class UserNetwork(object):
     def __init__(self, port):
         self._port = port
         
-    def settings_for(self, request):
-        forwarded_ports = _generate_forwarded_ports(request.public_ports)
+    def settings_for(self, image, request):
+        forwarded_ports = _generate_forwarded_ports(image, request.public_ports)
         socket_args = _generate_network_args(
             "guest-net-socket",
             "socket,mcast=230.0.0.1:{0},localaddr=127.0.0.1".format(self._port),
@@ -457,7 +466,7 @@ def _generate_network_args(name, netdev):
     ]
 
     
-def _generate_forwarded_ports(public_ports):
-    public_ports = set([_GUEST_SSH_PORT] + public_ports)
+def _generate_forwarded_ports(image, public_ports):
+    public_ports = set([image.ssh_internal_port] + public_ports)
     host_ports = starboard.find_local_free_tcp_ports(len(public_ports))
     return dict(zip(public_ports, host_ports))
