@@ -14,12 +14,12 @@ def remote_provider(url=None, hostname=None, port=None):
             url = "http://{0}:{1}/".format(hostname, port)
         else:
             raise TypeError("Must provide either: url, or; hostname and port")
-    return RemoteProvider(url)
+    return RemoteProvider(RemoteApi(url))
 
 
 class RemoteProvider(object):
-    def __init__(self, base_url):
-        self._api = RemoteApi(base_url)
+    def __init__(self, api):
+        self._api = api
         
     def start(self, *args, **kwargs):
         # TODO: remove duplication with QemuProvider.start
@@ -41,9 +41,6 @@ class RemoteProvider(object):
         machines = self._api.running_machines()
         return [_create_machine(machine, self._api) for machine in machines]
     
-    def _url(self, path):
-        return "{0}/{1}".format(self._base_url.rstrip("/"), path.lstrip("/"))
-    
     def __enter__(self):
         return self
         
@@ -60,12 +57,13 @@ class RemoteMachine(object):
         self.identifier = desc["identifier"]
         self.name = desc["name"]
         self.image_name = desc["imageName"]
-        self._hostname = desc["hostname"]
+        self.ssh_internal_port = desc["sshInternalPort"]
+        self._external_hostname = desc["externalHostname"]
         self._users = [dictobj.dict_to_obj(user, User) for user in desc["users"]]
         self._api = api
     
-    def hostname(self):
-        return self._hostname
+    def external_hostname(self):
+        return self._external_hostname
         
     def users(self):
         return self._users
